module Wing
  class Error < StandardError
    NET_HTTP_ERRORS = [
      EOFError,
      Errno::ECONNABORTED,
      Errno::ECONNREFUSED,
      Errno::ECONNRESET,
      Errno::EHOSTUNREACH,
      Errno::EINVAL,
      Errno::ENETUNREACH,
      Net::HTTPBadResponse,
      Net::HTTPHeaderSyntaxError,
      Net::ProtocolError,
      Net::OpenTimeout,
      Net::ReadTimeout,
      SocketError,
      Zlib::GzipFile::Error,
      OpenSSL::SSL::SSLError
    ]
  end

  # Raised on errors in the 400-499 range
  class ResponseError < Error; end

  # Raised when the API returns a 400 HTTP status code
  class BadRequestError < ResponseError; end

  # Raised when the API returns a 401 HTTP status code
  class AuthenticationError < ResponseError; end

  # Raised when the API returns a 404 HTTP status code
  class NotFoundError < ResponseError; end

  # Raised on errors in the 500-599 range
  class ServerError < Error; end

  # Raised when the API returns a 500 HTTP status code
  class InternalServerError < ServerError; end

  # Raised when we can't establish a connection to the API or if reading the reponse times out
  class ConnectionError < Error; end
end
